@@ -1,12 +1,20 @@
 #!/bin/bash
-# Copyright (C) 2000-2005 SWsoft. All rights reserved.
+#  Copyright (C) 2000-2007 SWsoft. All rights reserved.
 #
-# This file may be distributed under the terms of the Q Public License
-# as defined by Trolltech AS of Norway and appearing in the file
-# LICENSE.QPL included in the packaging of this file.
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  You should have received a copy of the GNU General Public License
+#  along with this program; if not, write to the Free Software
+#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
-# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
 # This script runnning on HN and perform next postinstall tasks:
 # 1. Randomizes crontab for given VE so all crontab tasks
@@ -21,7 +29,7 @@ function randcrontab()
 	[ -f "${file}" ] || return 0
 
 	/bin/cp -fp ${file} ${file}.$$
-	cat ${file} | /bin/awk '
+	cat ${file} | awk '
 BEGIN { srand(); }
 {
 	if ($0 ~ /^[ \t]*#/ || $0 ~ /^[ \t]+*$/) {
@@ -35,6 +43,9 @@ BEGIN { srand(); }
 	# min
 	if (ar[1] ~ /^[0-9]+$/) {
 		ar[1] = int(rand() * 59);
+	} else if (ar[1] ~/^-\*\/[0-9]+$/) {
+		r = int(rand() * 40) + 15;
+		ar[1] = "-*/" r;
 	}
 	# hour
 	if (ar[2] ~ /^[0-9]+$/) {
