@@ -42,6 +42,8 @@
 #define BACKUP		0
 #define DESTR		1
 
+const char destroy_dir_magic[]="vzctl-rm-me.";
+
 static int destroydir(char *dir);
 
 int del_dir(char *dir)
@@ -60,7 +62,7 @@ int del_dir(char *dir)
 
 int vps_destroy_dir(envid_t veid, char *dir)
 {
-        int ret;
+	int ret;
 
 	if (!quota_ctl(veid, QUOTA_STAT)) {
 		if ((ret = quota_off(veid, 0)))
@@ -70,7 +72,7 @@ int vps_destroy_dir(envid_t veid, char *dir)
 	quota_ctl(veid, QUOTA_DROP);	
 	if ((ret = destroydir(dir)))
 		return ret;
-        return 0;
+	return 0;
 }
 
 static char *get_destroy_root(char *dir)
@@ -110,25 +112,28 @@ static char *get_destroy_root(char *dir)
 
 char *maketmpdir(const char *dir)
 {
-        char buf[STR_SIZE];
-        char *tmp;
-        char *tmp_dir;
+	char buf[STR_SIZE];
+	char *tmp;
+	char *tmp_dir;
 	int len;
 
-        snprintf(buf, sizeof(buf), "%s/XXXXXXX", dir);
+	snprintf(buf, sizeof(buf), "%s/%sXXXXXXX", dir, destroy_dir_magic);
 	if ((tmp = mkdtemp(buf)) == NULL) {
 		logger(-1, errno, "Error in mkdtemp(%s)", buf);
 		return NULL;
 	}
 	len = strlen(dir);
-        tmp_dir = (char *)malloc(strlen(tmp) - len);
+	tmp_dir = (char *)malloc(strlen(tmp) - len);
 	if (tmp_dir == NULL)
 		return NULL;
-        strcpy(tmp_dir, tmp + len + 1);
+	strcpy(tmp_dir, tmp + len + 1);
 
-        return tmp_dir;
+	return tmp_dir;
 }
 
+/* Removes all the directories under 'root'
+ * those names start with 'destroy_dir_magic'
+ */
 static void _destroydir(char *root)
 {
 	char buf[STR_SIZE];
@@ -142,15 +147,15 @@ static void _destroydir(char *root)
 			return;
 		del = 0;
 		while ((ep = readdir(dp))) {
-			if (!strcmp(ep->d_name, ".") ||
-				!strcmp(ep->d_name, ".."))
+			if (strncmp(ep->d_name, destroy_dir_magic,
+					sizeof(destroy_dir_magic) - 1))
 			{
 				continue;
 			}
 			snprintf(buf, sizeof(buf), "%s/%s", root, ep->d_name);
 			if (stat(buf, &st)) 
 				continue;
-                	if (!S_ISDIR(st.st_mode))
+			if (!S_ISDIR(st.st_mode))
 				continue;
 			snprintf(buf, sizeof(buf), "rm -rf %s/%s",
 				root, ep->d_name);
@@ -194,7 +199,7 @@ static int destroydir(char *dir)
 		logger(-1, 0, "Unable to get root for %s", dir);
 		return -1;
 	}
-	snprintf(tmp, sizeof(buf), "%s/tmp", root);
+	snprintf(tmp, sizeof(tmp), "%s/vztmp", root);
 	free(root);
 	if (!stat_file(tmp)) {
 		if (mkdir(tmp, 0755)) {
@@ -202,17 +207,28 @@ static int destroydir(char *dir)
 			return VZ_FS_DEL_PRVT;
 		}
 	}
-	/* First move to del */
+	/* Fast/async removal -- first move to tmp dir */
 	if ((tmp_nm = maketmpdir(tmp)) == NULL)	{
 		logger(-1, 0, "Unable to generate temporary name in %s", tmp);
 		return VZ_FS_DEL_PRVT;
 	}
-	snprintf(buf, sizeof(tmp), "%s/%s", tmp, tmp_nm);
+	snprintf(buf, sizeof(buf), "%s/%s", tmp, tmp_nm);
 	free(tmp_nm);
 	if (rename(dir, buf)) {
-		logger(-1, errno, "Can't move %s -> %s", dir, buf);
 		rmdir(buf);
-		return VZ_FS_DEL_PRVT;
+		if (errno == EXDEV) {
+			/* See http://bugzilla.openvz.org/457 */
+			logger(0, 0, "Warning: directory %s is not on the same"
+					" filesystem as %s - doing slow/sync"
+					" removal", dir, tmp);
+			if (del_dir(dir))
+				return VZ_FS_DEL_PRVT;
+			else
+				return 0;
+		} else {
+			logger(-1, errno, "Can't move %s -> %s", dir, buf);
+			return VZ_FS_DEL_PRVT;
+		}
 	}
 	snprintf(buf, sizeof(buf), "%s/rm.lck", tmp);
 	if ((fd_lock = _lock(buf, 0)) == -2) {
@@ -267,7 +283,7 @@ int vps_destroy(vps_handler *h, envid_t veid, fs_param *fs)
 	}
 	if (vps_is_mounted(fs->root)) {
 		logger(0, 0, "VE is currently mounted (umount first)");
-                return VZ_FS_MOUNTED;
+		return VZ_FS_MOUNTED;
 	}
 	logger(0, 0, "Destroying VE private area: %s", fs->private);
 	if ((ret = vps_destroy_dir(veid, fs->private)))
