@@ -28,6 +28,7 @@
 #include <sys/ioctl.h>
 #include <linux/vzcalluser.h>
 #include <sys/personality.h>
+#include <linux/reboot.h>
 
 #include "vzerror.h"
 #include "res.h"
@@ -114,7 +115,7 @@ int vz_env_create_ioctl(vps_handler *h, envid_t veid, int flags)
 }
 
 /*
- * Reset standart file descriptord to /dev/null in case they are closed.
+ * Reset standard file descriptors to /dev/null in case they are closed.
  */
 static int reset_std()
 {
@@ -145,23 +146,23 @@ vps_handler *vz_open(envid_t veid)
 	vps_handler *h = NULL;
 
 	stdfd = reset_std();
-        if ((vzfd = open(VZCTLDEV, O_RDWR)) < 0) {
-                logger(-1, errno, "Unable to open %s", VZCTLDEV);
-                logger(-1, 0, "Please check that vzdev kernel module is loaded"
-                        " and you have sufficient permissions"
-                        " to access the file.");
+	if ((vzfd = open(VZCTLDEV, O_RDWR)) < 0) {
+		logger(-1, errno, "Unable to open %s", VZCTLDEV);
+		logger(-1, 0, "Please check that vzdev kernel module is loaded"
+			" and you have sufficient permissions"
+			" to access the file.");
 		goto err;
-        }
+	}
 	h = calloc(1, sizeof(*h));
 	if (h == NULL) 
 		goto err;
 	h->vzfd = vzfd;
 	h->stdfd = stdfd;
-        if (vz_env_create_ioctl(h, 0, 0) < 0 &&
+	if (vz_env_create_ioctl(h, 0, 0) < 0 &&
 		(errno == ENOSYS || errno == EPERM))
-        {
-                logger(-1, 0, "Your kernel lacks support for virtual"
-                                " environments or modules not loaded");
+	{
+		logger(-1, 0, "Your kernel lacks support for virtual"
+			" environments or modules not loaded");
 		goto err;
 	}
 	return h;
@@ -210,7 +211,7 @@ int vps_is_run(vps_handler *h, envid_t veid)
 		return 0;
 	else if (ret < 0)
 		logger(-1, errno, "error on vz_env_create_ioctl(VE_TEST)");
-        return 1;
+	return 1;
 }
 
 /** Change root to specified directory
@@ -250,14 +251,14 @@ int vz_chroot(char *root)
 
 int vz_setluid(envid_t veid)
 {
-        if (setluid(veid) == -1) {
+	if (setluid(veid) == -1) {
 		if (errno == ENOSYS)
 			logger(-1, 0, "Error: kernel does not support"
 				" user resources. Please, rebuild with"
 				" CONFIG_USER_RESOURCE=y");
-                return VZ_SETLUID_ERROR;
+		return VZ_SETLUID_ERROR;
 	}
-        return 0;
+	return 0;
 }
 
 /*
@@ -318,10 +319,12 @@ static int _env_create(vps_handler *h, envid_t veid, int wait_p, int err_p,
 		create_param.feature_mask |= VE_FEATURE_SYSFS;
 		create_param.known_features |= VE_FEATURE_SYSFS;
 	}
-	logger(3, 0, "Set features maks %016Lx/%016Lx",
+	logger(3, 0, "Set features mask %016Lx/%016Lx",
 			create_param.feature_mask,
 			create_param.known_features);
-	/* Close all fds  except std */
+
+	/* Close all fds except stdin. stdin is status pipe */
+	close(STDERR_FILENO); close(STDOUT_FILENO);
 	close_fds(0, wait_p, err_p, h->vzfd, -1);
 try:
 	ret = vz_env_create_data_ioctl(h, &env_create_data);
@@ -378,7 +381,7 @@ try:
 	*/
 	close(STDIN_FILENO);
 	/* Now we wait until VE setup will be done
-	   If no error start init otherwise exit.
+	   If no error, then start init, otherwise exit.
 	*/
 	if (read(wait_p, &ret, sizeof(ret)) != 0)
 		return 0;
@@ -404,7 +407,7 @@ static int vz_real_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		return ret;
 	if ((ret = setup_resource_management(h, veid, res)))
 		return ret;
-	/* Create another process to proper resource accounting */
+	/* Create another process for proper resource accounting */
 	if ((pid = fork()) < 0) {
 		logger(-1, errno, "Unable to fork");
 		return VZ_RESOURCE_ERROR;
@@ -438,10 +441,10 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 		return VZ_RESOURCE_ERROR;
 	}
 	sigaction(SIGCHLD, NULL, &actold);
-       	sigemptyset(&act.sa_mask);
-        act.sa_handler = SIG_IGN;
-       	act.sa_flags = SA_NOCLDSTOP;
-        sigaction(SIGCHLD, &act, NULL);
+	sigemptyset(&act.sa_mask);
+	act.sa_handler = SIG_IGN;
+	act.sa_flags = SA_NOCLDSTOP;
+	sigaction(SIGCHLD, &act, NULL);
 
 	if ((pid = fork()) < 0) {
 		logger(-1, errno, "Can not fork");
@@ -463,7 +466,7 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 			write(STDIN_FILENO, &ret, sizeof(ret));
 		exit(ret);
 	}
-	/* Wait for enviroment created */
+	/* Wait for environment created */
 	close(status_p[1]);
 	close(wait_p[0]);
 	close(err_p[1]);
@@ -490,7 +493,7 @@ int vz_env_create(vps_handler *h, envid_t veid, vps_res *res,
 				" wait functionality");
 			break;
 		}
-        }
+	}
 err:
 	close(status_p[1]);
 	close(status_p[0]);
@@ -605,34 +608,34 @@ int vps_start_custom(vps_handler *h, envid_t veid, vps_param *param,
 err:
 	free_dist_actions(&actions);
 	if (ret) {
-		/* Kill enviroment */
+		/* Kill environment */
 		logger(-1, 0, "VE start failed");
 		write(wait_p[1], &err, sizeof(err));
 	} else {
 		if (!read(err_p[0], &ret, sizeof(ret))) {
 			if (res->misc.wait == YES) {
 				logger(0, 0, "VE start in progress"
-					" waiting ...");
+					", waiting ...");
 				err = vps_execFn(h, veid, res->fs.root,
 					wait_on_fifo, NULL, 0);
 				if (err) {
 					logger(0, 0, "VE wait failed %s",
 						err == VZ_EXEC_TIMEOUT ? \
-						"timeout expired" : "");
+						"- timeout expired" : "");
 					ret = VZ_WAIT_FAILED;
 				} else {
 					logger(0, 0, "VE started"
 						" successfully");
 				}
 			} else {
-	                        logger(0, 0, "VE start in progress...");
+				logger(0, 0, "VE start in progress...");
 			}
 		} else {
 			if (ret == VZ_FS_BAD_TMPL)
 				logger(-1, 0, "Unable to start init, probably"
 					" incorrect template");
-                        logger(-1, 0, "VE start failed");
-                }
+			logger(-1, 0, "VE start failed");
+		}
 	}
 	if (ret) {
 		if (vps_is_run(h, veid))
@@ -655,7 +658,7 @@ err:
  * @param h		VE handler.
  * @param veid		VE id.
  * @param res		VE resourses.
- * @param d_actions	distribution specific sctions.
+ * @param d_actions	distribution specific actions.
  * @param skip		flags to skip VE setup (SKIP_SETUP|SKIP_ACTION_SCRIPT)
  * @param action	modules list, used to call setup() callback
  * @return		0 on success.
@@ -698,9 +701,9 @@ static int real_env_stop(vps_handler *h, envid_t veid, char *vps_root,
 		}
 		case M_KILL:
 		{
-			kill(-1, SIGTERM);
-			sleep(1);
-                        kill(1, SIGKILL);
+			syscall(__NR_reboot, LINUX_REBOOT_MAGIC1,
+					LINUX_REBOOT_MAGIC2,
+					LINUX_REBOOT_CMD_POWER_OFF, NULL);
 			break;
 		}
 	}
@@ -716,7 +719,7 @@ static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode)
 	sigemptyset(&act.sa_mask);
 	act.sa_handler = SIG_IGN;
 	act.sa_flags = SA_NOCLDSTOP;
-        sigaction(SIGCHLD, &act, NULL);
+	sigaction(SIGCHLD, &act, NULL);
 
 	logger(0, 0, "Stopping VE ...");
 	if (stop_mode == M_KILL)
@@ -738,7 +741,7 @@ static int env_stop(vps_handler *h, envid_t veid, char *root, int stop_mode)
 	}
 
 kill_vps:
-        if ((pid = fork()) < 0) {
+	if ((pid = fork()) < 0) {
 		ret = VZ_RESOURCE_ERROR;
 		logger(-1, errno, "Can not fork");
 		goto err;
@@ -770,8 +773,8 @@ err:
  * @param h		VE handler.
  * @param veid		VE id.
  * @param res		VE resourses.
- * @param stop_mode	stop mode one of (M_REBOOT M_HALT M_KILL).
- * @param skip		flags to skip run action script (SKIP_ACTION_SCRIPT)
+ * @param stop_mode	stop mode, one of (M_REBOOT M_HALT M_KILL).
+ * @param skip		flag to skip run action script (SKIP_ACTION_SCRIPT)
  * @param action	modules list, used to call cleanup() callback.
  * @return		0 on success.
  */
@@ -806,7 +809,7 @@ int vps_stop(vps_handler *h, envid_t veid, vps_param *param, int stop_mode,
 	mod_cleanup(h, veid, action, param);
 	vps_cleanup_res(h, veid, param, STATE_STOPPING);
 	ret = vps_umount(h, veid, res->fs.root, skip);
-	/* Clear VE network configuration*/
+	/* Clear VE network configuration */
 	ret = run_pre_script(veid, VPS_STOP);
 
 end:
