@@ -144,8 +144,8 @@ if (x != NULL) { 							\
 #define CHECK_B(name)							\
 if (ub->name != NULL) {							\
 	if ((ub->name[0] != ub->name[1])) {				\
-		logger(-1, 0, "Error: barrier should be equal limit for " \
-			#name " (currently, %lu:%lu)",			\
+		logger(-1, 0, "Error: barrier should be equal to limit" \
+			" for " #name " (currently, %lu:%lu)",		\
 			ub->name[0], ub->name[1]);			\
 		if (ask || recover) {					\
 			tmp_val0 = max_ul(ub->name[0], ub->name[1]); 	\
@@ -278,6 +278,25 @@ if (ub->name != NULL) {							\
 	CHECK_BL(param->dq.diskinodes, diskinodes)
 
 /*	2 Check formulas			*/
+	val = ub->numfile[0] * 384;
+	val &= LONG_MAX;
+	if (ub->dcachesize[1] < val) {
+		logger(-1, 0, "Warning: dcachesize.lim should be > %lu"
+				" (currently, %lu)", val,
+				ub->dcachesize[1]);
+		if (ask || recover) {
+			SET_MES(val);
+			if (ask)
+				recover = read_yn();
+			if (recover) {
+				ub->dcachesize[1] = val;
+				changed++;
+			}
+		}
+		if (!recover) ret = 1;
+//		if (!ask) fprintf(stderr, "\n");
+	}
+
 	val = (40 * 1024 * avnumproc) + ub->dcachesize[1];
 	val &= LONG_MAX;
 	if (ub->kmemsize[0] < val) {
@@ -521,24 +540,6 @@ if (ub->name != NULL) {							\
 		if (!recover) ret = 1;
 //		if (!ask) fprintf(stderr, "\n");
 	}
-	val = ub->numfile[0] * 384;
-	val &= LONG_MAX;
-	if (ub->dcachesize[1] < val) {
-		logger(-1, 0, "Warning: dcachesize.lim should be > %lu"
-				" (currently, %lu)", val,
-				ub->dcachesize[1]);
-		if (ask || recover) {
-			SET_MES(val);
-			if (ask)
-				recover = read_yn();
-			if (recover) {
-				ub->dcachesize[1] = val;
-				changed++;
-			}
-		}
-		if (!recover) ret = 1;
-//		if (!ask) fprintf(stderr, "\n");
-	}
 
 	return ret;
 }
