@@ -26,6 +26,17 @@
 #include <unistd.h>
 
 #include "types.h"
+#include "config.h"
+
+/* The below two lines are needed to link vzsplit with libvzctl */
+#include <logger.h>
+LOG_DATA
+
+/* #include <linux/magic.h>
+ * Instead of including a file which is not present in all environments,
+ * we copy-paste reiserfs magic from it
+ */
+#define REISERFS_SUPER_MAGIC    0x52654973
 
 #define SYSRSRV		52428800
 #define MEMPERVE	5542912
@@ -60,7 +71,7 @@
 #define FLOCK_PVE	1000
 #define PTY_PVE		512
 
-/* default discspace values */
+/* default diskspace values */
 #define HOST_DS		10737418240ULL /* 10 GB */
 #define DEF_DS		225280
 #define HOST_DI		100000
@@ -145,7 +156,7 @@ struct par_limits params[NUMUBC];
 /* Global variables */
 unsigned long long mem_total, low_total, swap_total, ds_total, di_total; 
 long pagesize, proc_calc;
-int num_ve, osl;
+int num_ve, ve_allowed, osl;
 
 float	k_kmem[MAX_SL]		= {1, 1.2, 1.8};
 float	k_nproc[MAX_SL]		= {1, 1.5, 2};
@@ -181,8 +192,11 @@ void header(FILE *fp)
 		"# on HN with total amount of physical mem %llu Mb\n"
 		"# low memory %llu Mb, swap size %llu Mb, Max treads %lu\n"
 		"# Resourse commit level %d:\n# %s\n",
-		 num_ve, (mem_total >> 20), (low_total >> 20), (swap_total >> 20),
-		 proc_calc, osl, level_string[osl]);
+		num_ve,
+		(mem_total >> 20),
+		(low_total >> 20), (swap_total >> 20), proc_calc,
+		osl, level_string[osl]
+		);
 	return;
 }
 
@@ -288,8 +302,8 @@ int calculate_values()
 
 	if (osl == MAX_SL) {
 		sl = osl - 1;
-                numproc = k_nproc[sl] * proc_calc / num_ve;
-                low_pve = low_total * 0.4 * K_KMEM_MAX / num_ve;
+		numproc = k_nproc[sl] * proc_calc / num_ve;
+		low_pve = low_total * 0.4 * K_KMEM_MAX / num_ve;
 		guarpg = (tot_pve - low_pve) / pagesize;
 		if  (guarpg < MIN_GUARPG) {
 			guarpg = MIN_GUARPG;
@@ -297,8 +311,8 @@ int calculate_values()
 		}
 		do {
 			numproc /= 2;
-                	if (numproc < MINPROC)
-                	        numproc = MINPROC;
+			if (numproc < MINPROC)
+				numproc = MINPROC;
 			avnproc = numproc / 2;
 			delta = MINSOCKBUF * numproc;
 			sockbuf = 2 * (MINDGRAMBUF + MINTCPBUF + delta);
@@ -404,6 +418,117 @@ int calculate_values()
 	return 0;
 }
 
+char * get_ve_private()
+{
+	vps_param *param;
+	char *ve_private, *veidp, *ret;
+
+	param = init_vps_param();
+	/* Parse global config file */
+	vps_parse_config(0, GLOBAL_CFG, param, NULL);
+	ve_private = param->res.fs.private_orig;
+
+	if (ve_private == NULL)
+	{
+		free_vps_param(param);
+		return NULL;
+	}
+
+	/* Remove $VEID and beyond from the string */
+	veidp = strstr(ve_private, "$VEID");
+	if (veidp == NULL)
+		veidp = strstr(ve_private, "${VEID}");
+	if (veidp != NULL)
+		*veidp = '\0';
+
+	ret = strdup(ve_private);
+	free_vps_param(param);
+	return ret;
+}
+
+int check_disk_space() {
+	char * ve_private;
+	int nofs = 0, noinodes = 0;
+	long ve_ds, ve_di;
+	int rec = 0, retval = 0;
+	struct statfs statfs_buf;
+
+	ve_private = get_ve_private();
+	if (ve_private == NULL) {
+		fprintf(stderr, "WARNING: unable to get VE_PRIVATE value "
+				"from " GLOBAL_CFG ".\n");
+		nofs = 1;
+	}
+	else {
+		if (statfs (ve_private, &statfs_buf) < 0) {
+			fprintf(stderr, "WARNING: statfs on %s failed: %s.\n",
+				ve_private, strerror(errno));
+			nofs = 1;
+		}
+	}
+
+	if (nofs == 1) {
+		fprintf(stderr, "Default disk space values to be used.\n\n");
+		ds_total = 0; di_total = 0;
+		return retval;
+	}
+
+	ds_total = statfs_buf.f_blocks;
+	di_total = statfs_buf.f_files;
+
+	if (statfs_buf.f_type == REISERFS_SUPER_MAGIC) {
+		/* reiserfs does not have inodes, thus
+		 * no limit on number of files */
+		noinodes = 1;
+	}
+
+	if (ds_total / 2 < HOST_DS / statfs_buf.f_bsize) {
+		rec = 1;
+		ds_total /= 2;
+	} else
+		ds_total -= HOST_DS / statfs_buf.f_bsize;
+
+	if (noinodes != 1) {
+		if (di_total / 2 < HOST_DI) {
+			rec = 1;
+			di_total /= 2;
+		} else
+			di_total -= HOST_DI;
+	}
+	if (rec)
+		fprintf(stderr, "WARNING: Recommended minimal size "
+				"of partition holding %s is 20Gb!\n",
+				ve_private);
+
+	ve_ds = ds_total / (DEF_DS);
+	ve_di = di_total / (DEF_DI);
+
+	if (ve_ds < num_ve) {
+		retval = 1;
+		ve_allowed = ve_ds;
+	}
+
+	if ((noinodes != 1) && (ve_di < num_ve) ) {
+		retval = 1;
+		if (ve_di < ve_ds)
+			ve_allowed = ve_di;
+	}
+	if (retval == 1) {
+		fprintf(stderr, "WARNING: partition holding %s do not "
+				"have space required for %d VEs\n"
+				"The maximum allowed value is %d\n",
+				ve_private, num_ve, ve_allowed);
+		fprintf(stderr, "Default disk space values "
+				"will be used\n\n");
+		ds_total = 0; di_total = 0;
+	}
+
+	free(ve_private);
+	return retval;
+}
+
+
+
 int main(int argc, char **argv)
 {
 	int len, opt, swp;
@@ -413,8 +538,7 @@ int main(int argc, char **argv)
 	FILE *fd;
 	char str[1024];
 	unsigned long long val;
-	struct statfs statfs_buf;
-	int retval, ve_allowed;
+	int retval;
 
 	num_ve = -1;
 	swp = 0;
@@ -426,7 +550,7 @@ int main(int argc, char **argv)
 		switch(opt) {
 		case 'f':
 			len = strlen(optarg) + strlen(VPS_CONF_DIR) +
-			       strlen("ve-.conf-sample");
+				strlen("ve-.conf-sample");
 			name = (char*)malloc(len + 1);
 			sprintf(name, VPS_CONF_DIR "ve-%s.conf-sample", optarg);
 			if (!stat(name, &st)) {
@@ -438,8 +562,8 @@ int main(int argc, char **argv)
 		case 'n':
 			num_ve = strtol(optarg, &tail, 10);
 			if (*tail != '\0') {
-				fprintf(stderr, "Invalid argument for -n: %s\n",
-					optarg);
+				fprintf(stderr, "Invalid argument "
+						"for -n: %s\n",	optarg);
 				exit(1);
 			}
 			break;
@@ -452,8 +576,8 @@ int main(int argc, char **argv)
 			swap_total = strtoll(optarg, &tail, 10);
 			swap_total <<= 10;
 			if (*tail != '\0') {
-				fprintf(stderr, "Invalid argument for -s: %s\n",
-						optarg);
+				fprintf(stderr, "Invalid argument "
+						"for -s: %s\n",	optarg);
 				exit(1);
 			}
 			break;
@@ -511,7 +635,7 @@ int main(int argc, char **argv)
 				"%d VEs can not be allocated\n", 
 				(mem_total + swap_total) >> 20, num_ve);
 		ve_allowed = (mem_total + swap_total - SYSRSRV) / MEMPERVE;
-                retval = 1;
+		retval = 1;
 	}
 
 	if (((low_total - SYSRSRV)/ ve_allowed) < LOWPERVE) {
@@ -527,7 +651,8 @@ int main(int argc, char **argv)
 	}
 
 	if (retval != 0) {
-		fprintf(stderr, "The maximum allowed value is %d\n", ve_allowed);
+		fprintf(stderr, "The maximum allowed value is %d\n",
+				ve_allowed);
 		exit(retval);
 	}
 
@@ -540,54 +665,8 @@ int main(int argc, char **argv)
 			proc_calc = val;
 	fclose(fd);
 
-	if (statfs ("/vz", &statfs_buf) < 0) {
-		fprintf(stderr, "WARNING: statfs /vz return error\n");
-		fprintf(stderr, "Therefore default disk values will be used\n\n");
-		ds_total = 0; di_total = 0;
-	} else {
-		long ve_ds, ve_di; 
-		long rec;
+	retval = check_disk_space();
 
-		ds_total = statfs_buf.f_blocks; 
-		di_total = statfs_buf.f_files; 
-
-		rec = 0;
-		if (ds_total / 2 < HOST_DS / statfs_buf.f_bsize) {
-			rec = 1;
-			ds_total /= 2;
-		} else
-			ds_total -= HOST_DS / statfs_buf.f_bsize;
-		if (di_total / 2 < HOST_DI) {
-			rec = 1;
-			di_total /= 2;
-		} else
-			di_total -= HOST_DI;
-		if (rec)
-			fprintf(stderr, "WARNING: Recommended minimal size"
-					"of /vz partition is 20Gb!\n");
-
-		ve_ds = ds_total / (DEF_DS);
-		ve_di = di_total / (DEF_DI);
-		
-		if (ve_ds < num_ve) {
-			retval = 1;
-			ve_allowed = ve_ds;
-		}
-		if (ve_di < num_ve) {
-			retval = 1;
-			if (ve_di < ve_ds)
-				ve_allowed = ve_di;
-		}
-		if (retval == 1) {
-			fprintf(stderr, "WARNING: /vz partition do not have space "
-				        "required for %d VEs\n"
-				       	"The maximum allowed value is %d\n",
-				       	num_ve, ve_allowed);
-			fprintf(stderr, "Default disc space values will be used\n\n");
-			ds_total = 0; di_total = 0;
-		/*	exit(retval);*/
-		}
-	}
 	if ((pagesize = sysconf(_SC_PAGE_SIZE)) == -1)
 		pagesize = 4096;
 	
